﻿
package net.kaikoga.text {
	
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.events.TextEvent;
	
	/**
	* キーボードからのテキスト入力が行われた。
	* @author kaikoga
	*/
	public class TextDownEvent extends TextEvent {
		
		/**
		 * 入力したテキストの文字コード。あるいは、TextDownEventクラスの定数。
		 * 複数文字の文字列が入力された際、TextDownSprite.splitStrings = falseのときは不正確な値になる。
		 */
		public var charCode:int;
		/**
		 * テキストを入力するために押したキーのキーコード。コピペを行ったときは不正確な値になる。
		 * @see flash.events.KeyboardEvent
		 */
		public var keyCode:int;
		/**
		 * テキストを入力するために押したキーの場所。コピペを行ったときは不正確な値になる。
		 * @see flash.events.KeyboardEvent
		 */
		public var keyLocation:int;
		/**
		 * テキスト入力時、altキーが押されていればtrue。
		 * @see flash.events.KeyboardEvent
		 */
		public var altKey:Boolean = false;
		/**
		 * テキスト入力時、ctrlキーが押されていればtrue。
		 * @see flash.events.KeyboardEvent
		 */
		public var ctrlKey:Boolean = false;
		/**
		 * テキスト入力時、shiftキーが押されていればtrue。
		 * @see flash.events.KeyboardEvent
		 */
		public var shiftKey:Boolean = false;
		
		/**
		 * テキスト入力イベントを表す文字列定数。
		 */
		public static const TEXT_DOWN:String = "ss_text_down";
		/**
		 * テキスト入力イベントに伴うキー解放を表す文字列定数。
		 */
		public static const TEXT_UP:String = "ss_text_up";
		
		/**
		 * キーボードのBackspace入力を表す整数定数。
		 */
		public static const BACKSPACE:int = 0x08;
		/**
		 * キーボードのDelete入力を表す整数定数。
		 */
		public static const DELETE:int = 0x07;
		/**
		 * キーボードのタブ入力を表す整数定数。
		 */
		public static const TAB:int = 0x09;
		/**
		 * キーボードのEsc入力を表す整数定数。
		 */
		public static const ESCAPE:int = 0x1b;
		/**
		 * キーボードの上矢印入力を表す整数定数。
		 */
		public static const UP:int = 0x1c;
		/**
		 * キーボードの下矢印入力を表す整数定数。
		 */
		public static const DOWN:int = 0x1d;
		/**
		 * キーボードの左矢印入力を表す整数定数。
		 */
		public static const LEFT:int = 0x1e;
		/**
		 * キーボードの右矢印入力を表す整数定数。
		 */
		public static const RIGHT:int = 0x1f;
		
		/**
		 * コンストラクタ。
		 * @param	type
		 * @param	bubbles
		 * @param	cancelable
		 * @param	charCodeValue
		 * @param	keyCodeValue
		 * @param	keyLocationValue
		 * @param	altKeyValue
		 * @param	ctrlKeyValue
		 * @param	shiftKeyValue
		 */
		public function TextDownEvent(type:String, bubbles:Boolean, cancelable:Boolean, charCodeValue:int,
				keyCodeValue:int, keyLocationValue:int,
				altKeyValue:Boolean, ctrlKeyValue:Boolean, shiftKeyValue:Boolean) {
			super(type, bubbles, cancelable, String.fromCharCode(charCodeValue));
			this.charCode = charCodeValue;
			this.keyCode = keyCodeValue;
			this.keyLocation = keyLocationValue;
			this.altKey = altKeyValue;
			this.ctrlKey = ctrlKeyValue;
			this.shiftKey = shiftKeyValue;
		}
		
		/**
		 * Eventの再送出を行う際に、オブジェクトの複製を生成する。
		 * @return このEventオブジェクトの複製。
		 */
		override public function clone():Event {
			return new TextDownEvent(this.type, this.bubbles, this.cancelable,
			this.charCode, this.keyCode, this.keyLocation,
			this.altKey, this.ctrlKey, this.shiftKey);
			
		}
		
		/**
		 * toStringの出力をオーバーライドする。
		 * @private
		 */
		override public function formatToString(className:String, ...arguments):String {
			arguments.unshift("TextDownEvent");
			arguments.push("charCode");
			arguments.push("keyCode");
			arguments.push("keyLocation");
			arguments.push("altKey");
			arguments.push("ctrlKey");
			arguments.push("shiftKey");
			return super.formatToString.apply(this, arguments);
		}
		
	}
	
}
