﻿
package net.kaikoga.fusalg.field {
	
	import flash.display.BitmapData;
	import flash.display.IBitmapDrawable;
	import flash.geom.Matrix;
	import flash.geom.Point;
	import flash.text.TextFormatAlign;
	import net.kaikoga.fusalg.area.AArea;
	import net.kaikoga.fusalg.area.AreaTitle;
	import net.kaikoga.fusalg.audio.AudioPlayer;
	import net.kaikoga.fusalg.cellinfo.FieldCellInfos;
	import net.kaikoga.fusalg.chip.AChip;
	import net.kaikoga.fusalg.chip.DotchipChipSet;
	import net.kaikoga.fusalg.debug.EDebug;
	import net.kaikoga.fusalg.layer.ALayer;
	import net.kaikoga.fusalg.layer.EndingLayer;
	import net.kaikoga.fusalg.layer.GameOverLayer;
	import net.kaikoga.fusalg.layer.HelpLayer;
	import net.kaikoga.fusalg.layer.MaskLayer;
	import net.kaikoga.fusalg.layer.MessageLayer;
	import net.kaikoga.fusalg.layer.TitleLayer;
	import net.kaikoga.fusalg.mortal.ACharacter;
	import net.kaikoga.fusalg.mortal.AMortal;
	import net.kaikoga.fusalg.mortal.ally.AHeroine;
	import net.kaikoga.fusalg.mortal.ally.Fusa;
	import net.kaikoga.fusalg.mortal.APart;
	import net.kaikoga.fusalg.mortal.bullet.ABullet;
	import net.kaikoga.fusalg.mortal.effects.ASimpleEffect;
	import net.kaikoga.fusalg.util.FusalgMath;
	import net.kaikoga.fusalg.util.KeyState;
	import net.kaikoga.fusalg.chip.Chips;
	
	/**
	 * ...
	 * @author kaikoga
	 */
	public class Field {
		
		public var area:AArea;
		public var rows:Array/*Row*/;
		public var bitmapData:BitmapData;
		
		public static const FIELD_ROWS:int = 12;
		public static const ROW_CELLS:int = 11;
		public static const CHIP_WIDTH:int = 32;
		public static const CHIP_HEIGHT:int = 32;

		public static const BOUND_X_MIN:int = -32;
		public static const BOUND_X_MAX:int = 384;
		public static const BOUND_Y_MIN:int = -32;
		public static const BOUND_Y_MAX:int = 448;
		public var camera:Point;
		public var drawBlink:Boolean = false;
		
		private static var workPt:Point = new Point();
		private static var workMatrix:Matrix = new Matrix(1, 0, 0, 1, 0, 0);
		
		public var titleLayer:TitleLayer;
		public var maskLayer:MaskLayer;
		public var messageLayer:MessageLayer;
		public var gameOverLayer:GameOverLayer;
		public var endingLayer:EndingLayer;
		public var helpLayer:HelpLayer;
		public var allLayers:Array;
		
		public var allMortals:Array;
		//shortcut; readonly
		public var avatar:Fusa;
		public var heroine:AHeroine;
		public var enemies:Array/*AMortal*/;
		public var effects:Array/*ASimpleEffect*/;
		
		public var keyState:KeyState;
		
		private var fontChips:DotchipChipSet;
		
		public var flags:Flags;
		
		public var nowavatarCell:Cell;
		
		public var audioPlayer:AudioPlayer;
		
		public function Field(bitmapdata:BitmapData, keyState:KeyState) {
			super();
			this.audioPlayer = new AudioPlayer();
			this.fontChips = Chips.getInstance().fontS;
			this.titleLayer = new TitleLayer();
			this.maskLayer = new MaskLayer();
			this.messageLayer = new MessageLayer();
			this.gameOverLayer = new GameOverLayer();
			this.endingLayer = new EndingLayer();
			this.helpLayer = new HelpLayer();
			this.camera = new Point(0, 0);
			this.allLayers = [];
			this.rows = [];
			var fieldcellinfos:FieldCellInfos = FieldCellInfos.getInstance()
			for (var i:int = 0; i < FIELD_ROWS; i++) {
				this.rows[i] = new Row(fieldcellinfos.groundSet1.vert, fieldcellinfos.sideSet.vert);
			}
			this.bitmapData = bitmapdata;
			
			this.avatar = new Fusa();
			this.avatar.position.x = ROW_CELLS * CHIP_WIDTH / 2;
			this.allMortals = [this.avatar];
			this.enemies = [];
			this.effects = [];
			this.keyState = keyState;
			this.enableLayer(this.messageLayer);
			this.enableLayer(this.maskLayer);
			this.enableLayer(this.helpLayer);
			this.initTitle(true, true);
		}
		
		public function initTitle(title:Boolean = true, gameover:Boolean = false):void {
			this.audioPlayer.stopChannel("music");
			if (title) {
				this.flags = new Flags();
				this.avatar.init();
				this.enableLayer(this.titleLayer);
			}
			if (gameover) {
				this.messageLayer.flush();
				for (var i:int = this.enemies.length - 1; i >= 0; i--) {
					var enemy:ACharacter = this.enemies[i] as ACharacter;
					if (enemy) {
						enemy.life = 4;
						enemy.onGetDamage(this, null, 4);
					}
				}
				this.setHeroine();
				if (!title) {
					this.enableLayer(this.gameOverLayer);
				}
			}
			if (!(this.area is AreaTitle)) {
				this.area = new AreaTitle();
			}
		}
		
		public function nextArea(area:AArea):void {
			area.onEnterSelf(this);
			this.area = area;
		}
		
		public function frameMove():void {
			this.flags.frameMove(this);
			if (EDebug.TRACE_MORTAL_COUNT) {
				trace("l=", this.allLayers.length, "e=", this.enemies.length, "m=", this.allMortals.length);
			}
			this.drawBlink = !this.drawBlink;
			var movemortals:Boolean = true;
			if (this.flags.hour >= 18) {
				switch (this.flags.minute) {
					case 0:
					case 1:
					case 2:
					movemortals = false;
					break;
					case 3:
					this.endingLayer.endType = "end5";
					this.enableLayer(this.endingLayer);
					this.flags.minute = 4;
					break;
				}
			}
			if (movemortals) {
				var i:int = 0;
				if (this.avatar) {
					var dy:int = 12 - (this.avatar.position.y >> 4);
					if (dy < 0) {
						dy = 0;
					}
					this.camera.y += dy;
					if (this.camera.y >= 0) {
						this.loadNextRow();
					}
					this.avatar.frameMove(this);
					var newcell:Cell = this.getCellAt(avatar.position.x, avatar.position.y);
					if (newcell != this.nowavatarCell) {
						if (this.nowavatarCell && this.nowavatarCell.event) {
							this.nowavatarCell.event.onExit(this);
						}
						if (newcell.event) {
							newcell.event.onEnter(this);
						}
						this.nowavatarCell = newcell;
					} else {
						if (newcell.event) {
							newcell.event.frameMove(this);
						}
					}
				}
				if (this.heroine) {
					this.heroine.frameMove(this);
				}
				for (i = this.enemies.length - 1; i >= 0; i--) {
					var enemy:AMortal = AMortal(this.enemies[i]);
					enemy.frameMove(this);
					if (this.avatar.checkHit(enemy)) {
						this.avatar.onHit(this, enemy);
					}
					if (enemy.checkHit(this.avatar)) {
						enemy.onHit(this, this.avatar);
					}
					if (enemy.willDispose()) {
						this.enemies.splice(i, 1);
						this.allMortals.splice(this.allMortals.indexOf(enemy), 1);
						enemy.onDispose(this);
					}
				}
			}
			for (i = this.effects.length - 1; i >= 0; i--) {
				var effect:ASimpleEffect = ASimpleEffect(this.effects[i]);
				effect.frameMove(this);
				if (effect.willDispose()) {
					this.effects.splice(i, 1);
					this.allMortals.splice(this.allMortals.indexOf(effect), 1);
					effect.onDispose(this);
				}
			}
			for (i = this.allLayers.length-1; i >= 0; i--) {
				if (!ALayer(this.allLayers[i]).frameMove(this)) {
					this.allLayers.splice(i, 1);
				}
			}
			this.drawSelf();
		}
		
		public function getCellAt(x:int, y:int):Cell {
			var gridx:int = x >> 5;
			var gridy:int = y >> 5;
			gridx = (gridx < 0) ? 0 : (gridx >= ROW_CELLS) ? ROW_CELLS-1: gridx;
			gridy = (gridy < 0) ? 0 : (gridy >= FIELD_ROWS) ? FIELD_ROWS-1: gridy;
			return this.rows[gridy].cells[gridx];
		}
		
		public function isEnterable(x:int, y:int):Boolean {
			return this.getCellAt(x, y).isEnterable;
		}
		
		public function getRandomEnemy():AMortal {
			return (this.enemies.length == 0) ? null : this.enemies[FusalgMath.randomInt(0, this.enemies.length)];
		}
		private function drawSelf():void {
			var i:int;
			var offsety:int = 0;
			for (i = 0; i < FIELD_ROWS; i++) {
				Row(this.rows[i]).drawSelf(this, 0, offsety);
				offsety += CHIP_HEIGHT;
			}
			for (i = this.enemies.length - 1; i >= 0; i--) {
				AMortal(this.enemies[i]).drawSelf(this);
			}
			for (i = this.effects.length - 1; i >= 0; i--) {
				ASimpleEffect(this.effects[i]).drawSelf(this);
			}
			if (this.heroine) {
				this.heroine.drawSelf(this);
			}
			if (this.avatar) {
				this.avatar.drawSelf(this);
			}
			for (i = 0; i < this.allLayers.length; i++) {
				ALayer(this.allLayers[i]).drawSelf(this);
			}
		}
		
		private function loadNextRow():void {
			var row:Row = this.rows.pop();
			this.rows.unshift(row);
			this.area.nextRow(this, row);
			this.camera.y -= CHIP_HEIGHT;
			for (var i:int = this.allMortals.length-1; i >= 0; i--) {
				AMortal(this.allMortals[i]).shift(0, CHIP_HEIGHT);
			}
		}
		
		private function tryPlaceCharacter(mortal:AMortal, x:int = -1, y:int = -32):void {
			mortal.position.y = y;
			//field width is 256
			mortal.position.x = (x >= 0) ? x : (CHIP_WIDTH + int(Math.random() * 256));
			//retry 16 times
			for (var i:int = 0; i < 16; i++) {
				if (this.isEnterable(mortal.position.x, mortal.position.y)) {
					break;
				}
				mortal.position.x = CHIP_WIDTH + int(Math.random() * 256);
			};
		}
		
		public function setHeroine(heroine:AHeroine = null, x:int = -1, y:int = -32):void {
			if (this.heroine) {
				this.allMortals.splice(this.allMortals.indexOf(this.heroine), 1);
			}
			this.heroine = heroine;
			if (heroine) {
				this.tryPlaceCharacter(heroine, x, y);
				this.allMortals.push(heroine);
			}
		}
		
		public function addEnemy(enemy:ACharacter, x:int = -1, y:int = -32):void {
			this.tryPlaceCharacter(enemy, x, y);
			enemy.onAdd(this);
			this.enemies.push(enemy);
			this.allMortals.push(enemy);
		}
		
		public function addPart(part:APart, x:int = 0, y:int = 0):void {
			part.moveRelative(x, y);
			part.onAdd(this);
			this.enemies.push(part);
			this.allMortals.push(part);
		}
		
		public function addBullet(bullet:ABullet, x:int = 0, y:int = 0):void {
			bullet.position.x = x;
			bullet.position.y = y;
			this.enemies.push(bullet);
			this.allMortals.push(bullet);
		}
		
		public function addEffect(effect:ASimpleEffect, x:int = -1, y:int = -32):void {
			effect.position.x = x;
			effect.position.y = y;
			this.effects.push(effect);
			this.allMortals.push(effect);
		}
		
		public function enableLayer(layer:ALayer):void {
			layer.onActivate(this);
			if (this.allLayers.indexOf(layer) < 0) {
				this.allLayers.push(layer);
			}
		}
		
		public function drawChip(chip:AChip, dir:int, x:int, y:int, global:Boolean = false):void {
			if (!chip) {
				return;
			}
			var pt:Point = workPt;
			if (global) {
				pt.x = x;
				pt.y = y;
			} else {
				pt.x = x + this.camera.x;
				pt.y = y + this.camera.y;
			}
			chip.copySelf(this.bitmapData, dir, pt);
		}
		
		public function drawString(value:String, x:int, y:int, align:String, global:Boolean = false):void {
			if (!value) {
				return;
			}
			var c:int = value.length;
			var cursorx:int = x;
			var cursory:int = y;
			switch (align) {
				case TextFormatAlign.RIGHT:
				cursorx -= c * 16;
				break;
			}
			for (var i:int = 0; i < c; i++) {
				var ch:int = value.charCodeAt(i);
				this.drawChip(this.fontChips.getChipByIndex(ch), 0, cursorx, cursory, global);
				cursorx += 16;
			}
		}

		public function drawEtc(drawable:IBitmapDrawable, x:int, y:int, global:Boolean = false):void {
			if (!drawable) {
				return;
			}
			var matrix:Matrix = workMatrix;
			if (global) {
				matrix.tx = x;
				matrix.ty = y;
			} else {
				matrix.tx = x + this.camera.x;
				matrix.ty = y + this.camera.y;
			}
			this.bitmapData.draw(drawable, matrix);
		}
	}
	
}
