﻿
package net.kaikoga.fusalg.mortal {
	
	import net.kaikoga.fusalg.audio.Sounds;
	import net.kaikoga.fusalg.chip.DotchipChipSet;
	import net.kaikoga.fusalg.debug.EDebug;
	import net.kaikoga.fusalg.field.Field;
	import net.kaikoga.fusalg.mortal.effects.SparkEffect;
	import net.kaikoga.fusalg.mortal.effects.SparkEffectL;
	
	/**
	 * ...
	 * @author kaikoga
	 */
	public class ACharacter extends AMortal {

		public var life:int = 1;
		public var level:int = 1;
		public var exp:int = 0;
		public var killExp:int = 0;
		public var killGold:int = 0;
		public var armor:int = 8;
		public var singleDir:Boolean = false;
		public var sparkType:String = "spark1";
		
		public var invincibleFrames:int = 0;
		protected var killSparkFrames:int = 0;
		
		public function ACharacter(chips:DotchipChipSet) {
			super(chips);
			
		}
		
		public function onGetDamage(field:Field, src:AMortal, damage:int):void {
			if (this.life > 0) {
				if (EDebug.TRACE_DAMAGE) {
					trace("ACharacter.onGetDamage():", src, "deals", this, damage, "pts of damage.");;
				}
				for (var i:int = (damage < this.life) ? damage : this.life; i > 0; i--) {
					field.addEffect(new SparkEffect(this.sparkType, this.dir ^ 2), this.position.x, this.position.y);
				}
				switch (this.sparkType) {
					case "spark1":
					field.audioPlayer.tryStartAudio(Sounds.getInstance().seAttack);
					break;
					case "spark2":
					field.audioPlayer.tryStartAudio(Sounds.getInstance().seDamage);
					break;
				}
				this.life -= damage;
				if (this.life <= 0) {
					if (src != null) {
						field.audioPlayer.tryStartAudio(Sounds.getInstance().seDefeat);
						field.addEffect(new SparkEffectL(), this.position.x, this.position.y);
						this.onKill(field);
						field.avatar.gainExp(field, this.killExp);
						field.flags.gold += this.killGold;
						if (field.heroine) {
							field.heroine.gainExp(this.killExp);
						}
					}
				}
			}
		}
		
		override public function onHit(field:Field, src:AMortal):void {
			if (this.life == 0) {
				return;
			}
			if (EDebug.TRACE_HIT) {
				trace(this, "hit by", src);
			}
			var damage:int = (src.isProjectile ? src.projectileDamage : src.physicalDamage);
			if (this.invincibleFrames < 0) {
				if (damage > 0) {
					this.invincibleFrames = this.armor;
				}
				this.onGetDamage(field, src, damage);
			} else {
				//this.invincibleFrames = 1;
			}
		}
		
		public function onKill(field:Field):void {
			for (var t:int = 0; t < this.killSparkFrames; t++) {
				field.addEffect(new SparkEffectL(-1, t), this.position.x, this.position.y);
			}
		}
		
		public function attackTerrain(field:Field):void {
			var x:int = this.position.x;
			var y:int = this.position.y;
			switch (this.dir) {
				case DIR_RIGHT:
				x += 24;
				break;
				case DIR_UP:
				y -= 24;
				break;
				case DIR_LEFT:
				x -= 24;
				break;
				case DIR_DOWN:
				y += 24;
				break;
			}
			if (field.getCellAt(x, y).takeDamage()) {
				for (var i:int = 0; i < 8; i++) {
					field.addEffect(new SparkEffect("spark1", this.singleDir ? -1 : this.dir), x + 16, y + 16);
				}
				field.audioPlayer.tryStartAudio(Sounds.getInstance().seTerrain);
			}
		}

		public function moveSelf(field:Field, dx:int, dy:int, dir:int = -1):Boolean {
			var result:Boolean = true;
			var nx:int = this.position.x + dx;
			var ny:int = this.position.y + dy;
			if (EDebug.DEBUG_AREA) {
				this.position.x = nx;
				this.position.y = ny;
			} else {
				if (field.isEnterable(nx, this.position.y)) {
					this.position.x = nx;
				} else {
					result = false;
				}
				if (field.isEnterable(this.position.x, ny)) {
					this.position.y = ny;
				} else {
					result = false;
				}
			}
			if (dir >= 0) {
				this.dir = dir;
			} else {
				if (((dx > 0) ? dx : -dx) > ((dy > 0) ? dy : -dy)) {
					if (dx > 0) {
						this.dir = DIR_RIGHT;
					} else if (dx < 0) {
						this.dir = DIR_LEFT;
					}
				} else {
					if (dy < 0) {
						this.dir = DIR_UP;
					} else if (dy > 0) {
						this.dir = DIR_DOWN;
					}
				}
			}
			return result;
		}

		public function yankBoss(d:int = 4):void {
			if (this.position.y > 320) {
				this.position.y -= d;
			} else if (this.position.y < 0) {
				this.position.y += d;
			}
			if (this.position.x > 320) {
				this.position.x -= d;
			} else if (this.position.x < 32) {
				this.position.x += d;
			}
		}
		
		protected function willDisposeByLife():Boolean {
			return ((this.life <= 0) && (this.invincibleFrames < 0))
		}
		
		override public function willDispose():Boolean {
			return (super.willDispose() || this.willDisposeByLife());
		}
		
		override public function frameMove(field:Field):void {
			this.invincibleFrames--;
		}
		
		override public function drawSelf(field:Field):void {
			if ((this.invincibleFrames < 0) || field.drawBlink) {
				super.drawSelf(field);
			}
		}

	}
	
}
