﻿
package net.kaikoga.fusalg.mortal {
	
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import net.kaikoga.fusalg.chip.AChip;
	import net.kaikoga.fusalg.chip.Chips;
	import net.kaikoga.fusalg.chip.DotchipChipSet;
	import net.kaikoga.fusalg.field.Cell;
	import net.kaikoga.fusalg.field.Field;
	
	/**
	 * ...
	 * @author kaikoga
	 */
	public class AWarrior extends ACharacter {
		
		private var type:String = null;
		private var state:String = "run";
		private var myProjectile:AWarriorProjectile;
		private var weaponChips:DotchipChipSet;
		
		private static const TYPE_CHIPNAMES:Object = {
			sword:["sword1", "sword2"],
			axe:["axe1", "axe2"],
			shield:["shield1", "shield2"]
		};
		
		private static const FRAME_GEOM:Object = {
			stop:[
				{x:6, y:0, fore:true},
				{x:6, y:0, fore:false},
				{x:-6, y:0, fore:false},
				{x:-6, y:0, fore:true}
			],
			run0:[
				{x:6, y:0, fore:true},
				{x:6, y:0, fore:false},
				{x:-6, y:0, fore:false},
				{x:-6, y:0, fore:true}
			],
			run1:[
				{x:10, y:0, fore:true},
				{x:4, y:-4, fore:false},
				{x:-10, y:0, fore:false},
				{x:-4, y:4, fore:true}
			],
			run2:[
				{x:2, y:-2, fore:true},
				{x:8, y:2, fore:false},
				{x:-2, y:-2, fore:false},
				{x:-8, y:-2, fore:true}
			],
			attack1:[
				{x:16, y:0, fore:true},
				{x:1, y:-16, fore:false},
				{x:-16, y:0, fore:false},
				{x:-1, y:16, fore:true}
			],
			attack2:[
				{x:12, y:3, fore:true},
				{x:0, y:-12, fore:false},
				{x:-12, y:3, fore:false},
				{x:0, y:12, fore:true}
			]
		};
			
		public function AWarrior(chips:DotchipChipSet) {
			super(chips);
			this.weaponChips = Chips.getInstance().weaponChipM;
			this.chipName = "stop";
			this.projectileBound = new Rectangle(-12, -20, 24, 24);
			this.physicalBound = this.projectileBound;
		}
		
		private static const RUN_NAMES:Array/*String*/= ["run0", "run1", "run0", "run2"];
		public function run(field:Field, dx:int, dy:int):Boolean {
			this.chipName = (dx | dy) ? RUN_NAMES[(this.frame >> 2) & 3] : "stop";
			return this.moveSelf(field, dx, dy);
		}
		
		public function canAttack():Boolean {
			return !this.myProjectile;
		}
		
		public function attack(field:Field, type:String, projectile:AWarriorProjectile):void {
			this.type = type;
			if (this.state == "run") {
				this.frame = 0;
				this.state = "attack1";
				this.myProjectile = projectile;
				projectile.dir = this.dir;
				if (type == "axe") {
					this.attackTerrain(field);
				}
			}
		}

		override public function frameMove(field:Field):void {
			this.frame++;
			switch (this.state) {
				case "attack1":
				this.chipName = "attack1";
				if (this.frame >= ((this.type == "shield") ? 12 : 4)) {
					this.state = "attack2";
					this.frame = 0;
				}
				break;
				case "attack2":
				this.chipName = "attack2";
				if (this.frame >= 4) {
					this.frame = 0;
					this.state = "run";
				}
				break;
			}
			super.frameMove(field);
			if (this.myProjectile) {
				if (this.myProjectile.willDispose()) {
					this.myProjectile = null;
				} else {
					this.myProjectile.position.x = this.position.x;
					this.myProjectile.position.y = this.position.y;
					this.myProjectile.frameMove(field);
				}
			}
		}
		
		protected function getMyProjectileDamage():int {
			return this.myProjectile ? this.myProjectile.projectileDamage : 0;
		}
		
		override public function drawSelf(field:Field):void {
			var typedata:Array= TYPE_CHIPNAMES[this.type];
			var chip:AChip;
			var geomdata:Object;
			if (typedata) {
				switch (this.state) {
					case "attack1":
					case "attack2":
					chip = this.weaponChips.getChipByName(typedata[1]);
					break;
					default:
					chip = this.weaponChips.getChipByName(typedata[0]);
					break;
				}
				var framedata:Array = FRAME_GEOM[this.chipName];
				if (framedata) {
					geomdata = framedata[this.dir];
				}
			}
			if (!geomdata) {
				super.drawSelf(field);
			} else if (geomdata.fore) {
				super.drawSelf(field);
				field.drawChip(chip, this.dir, this.position.x + geomdata.x - 16, this.position.y + geomdata.y - 24);
			} else {
				field.drawChip(chip, this.dir, this.position.x + geomdata.x - 16, this.position.y + geomdata.y - 24);
				super.drawSelf(field);
			}
			if (this.myProjectile) {
				this.myProjectile.drawSelf(field);
			}
		}
		
	}
	
}
