﻿
package net.kaikoga.fusalg.mortal.ally {
	
	import flash.geom.Rectangle;
	import net.kaikoga.fusalg.chip.DotchipChipSet;
	import net.kaikoga.fusalg.field.Field;
	import net.kaikoga.fusalg.mortal.ACharacter;
	import net.kaikoga.fusalg.mortal.AMortal;
	import net.kaikoga.fusalg.mortal.assist.AAssistMortal;
	import net.kaikoga.fusalg.strings.FusalgStrings;
	import net.kaikoga.fusalg.util.FusalgMath;
	
	/**
	 * ...
	 * @author kaikoga
	 */
	public class AHeroine extends ACharacter {
		
		private var targetX:int;
		private var targetY:int;
		private var flitFrames:int = 0;
		private var attackWait:int = 512;
		private var messageWait:int;
		private var isTooFar:Boolean = false;
		
		public var attackLevel:int = 1;
		public var auraLevel:int = 1;
		
		private var assists:Array/*AAssistMortal*/;
		
		protected var runSpeed:int = 3;
		protected var warpSpeed:int = 4;
		
		protected var chipNames:Array/*String*/;
		public var nameName:String;
		public var skillName:String;
		
		public function AHeroine(chips:DotchipChipSet, chipnames:Array/*String*/) {
			super(chips);
			this.chipNames = chipnames;
			this.chipName = "stop";
			this.physicalDamage = 1;
			this.physicalBound = new Rectangle(-14, -22, 28, 28);
			this.assists = [];
			this.messageWait = this.getNextMessageWait();
		}
		
		override public function willDispose():Boolean {
			//Never disposes
			return false;
		}
		
		protected function onLevelUp():void {
			
		}
		
		public function gainExp(value:int):void {
			value -= this.level;
			if (value < 1) {
				value = 1;
			}
			this.exp += value;
			if (this.exp >= 100) {
				this.exp -= 100;
				this.level++;
				this.physicalDamage++;
				this.onLevelUp();
			}
		}
		
		protected function getNextMessageWait():int {
			return 10000000;
		}
		
		protected function addAssist(field:Field, assist:AAssistMortal):void {
			assist.onAdd(field);
			this.assists.push(assist);
		}
		
		protected function canAssist():Boolean {
			return this.assists.length == 0;
		}
		
		public function assistAttack(field:Field):void {
			;
		}
		
		public function assistAura(field:Field):void {
			;
		}
		
		public function castMyHit(field:Field):void {
			for (var i:int = field.enemies.length - 1; i >= 0; i--) {
				var enemy:AMortal = AMortal(field.enemies[i]);
				if (enemy.checkHit(this)) {
					this.onHit(field, enemy);
					enemy.onHit(field, this);
				}
			}
		}
		
		public function run(field:Field, dx:int, dy:int):Boolean {
			this.frame++;
			if (dx | dy) {
				switch ((this.frame >> 2) & 3) {
					case 0:
					this.chipName = this.chipNames[2];
					break;
					case 2:
					this.chipName = this.chipNames[3];
					break;
					default:
					this.chipName = this.chipNames[1];
					break;
				}
			} else {
				this.chipName = this.chipNames[0];
			}
			return this.moveSelf(field, dx, dy);
		}
		
		override public function frameMove(field:Field):void {
			super.frameMove(field);
			if (--this.messageWait < 0) {
				this.messageWait = this.getNextMessageWait();
				var message:String = field.area.getAreaMessage(this.nameName);
				if (message) {
					field.messageLayer.message(field, FusalgStrings.getString(message));
				}
			}
			this.attackWait -= (this.attackLevel + 8);
			if (this.attackWait < 0) {
				this.assistAttack(field);
				this.attackWait = 512 - Math.random() * 256;
			}
			for (var i:int = this.assists.length - 1; i >= 0; i--) {
				var assistmortal:AAssistMortal = this.assists[i];
				assistmortal.frameMove(field);
				if (assistmortal.willDispose()) {
					this.assists.splice(i, 1);
				}
			}
			this.castMyHit(field);
			if (--this.flitFrames < 0) {
				this.targetX = FusalgMath.randomInt(16, 272);
				this.targetY = field.avatar.position.y + FusalgMath.randomInt( -32, +32);
				if (!field.isEnterable(this.position.x, this.position.y)) {
					//Stuck in the wall, so continue warping
					this.isTooFar = true;
					this.flitFrames = FusalgMath.randomInt(4, 8);
				} else {
					//Warp if too far from Fusa
					this.isTooFar = Math.abs(field.avatar.position.y - this.position.y) > 160;
					this.flitFrames = FusalgMath.randomInt(20, 50);
				}
			}
			var dx:int = 0;
			var dy:int = 0;
			var SPEED:int = this.isTooFar ? this.warpSpeed : this.runSpeed;
			if (this.targetX > this.position.x + 8) {
				dx = SPEED;
				this.dir = 0;
			} else if (this.targetX < this.position.x - 8) {
				dx = -SPEED;
				this.dir = 2;
			}
			if (this.targetY < this.position.y - 8) {
				dy = -SPEED;
				this.dir = 1;
			} else if (this.targetY > this.position.y + 8) {
				dy = SPEED;
				this.dir = 3;
			}
			if (this.isTooFar) {
				//Do warp
				this.position.x += dx;
				this.position.y += dy;
			} else {
				this.run(field, dx, dy);
			}
		}
		
		public function drawAssists(field:Field):void {
			for (var i:int = this.assists.length - 1; i >= 0; i--) {
				AMortal(this.assists[i]).drawSelf(field);
			}
		}
		
		override public function drawSelf(field:Field):void {
			this.drawAssists(field);
			super.drawSelf(field);
		}
		
		override public function shift(dx:int, dy:int):void {
			super.shift(dx, dy);
			for (var i:int = this.assists.length - 1; i >= 0; i--) {
				AMortal(this.assists[i]).shift(dx, dy);
			}
		}
		
	}
	
}
