﻿
package net.kaikoga.fusalg.mortal.ally {
	
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import net.kaikoga.fusalg.audio.Sounds;
	import net.kaikoga.fusalg.chip.Chips;
	import net.kaikoga.fusalg.debug.EDebug;
	import net.kaikoga.fusalg.field.Field;
	import net.kaikoga.fusalg.mortal.AMortal;
	import net.kaikoga.fusalg.mortal.AWarrior;
	import net.kaikoga.fusalg.mortal.effects.LevelUpEffect;
	import net.kaikoga.fusalg.mortal.effects.LevelUpMessageEffect;
	import net.kaikoga.fusalg.strings.FusalgStrings;
	import net.kaikoga.fusalg.util.FusalgMath;
	
	/**
	 * ...
	 * @author kaikoga
	 */
	public class Fusa extends AWarrior {
		
		public var swordLevel:int = 1;
		public var axeLevel:int = 1;
		public var shieldLevel:int = 1;
		
		private var aura:Aura;
		public var healWait:int;
		public var quickHealing:Boolean = false;
		public var maxLife:int;
		
		private static const PHYSICAL_BOUND:Rectangle = new Rectangle( -12, -22, 24, 28);
		private static const PROJECTILE_BOUND:Rectangle = new Rectangle( -2, -10, 4, 4);
		
		public function Fusa() {
			super(Chips.getInstance().avatarChipM);
			this.init();
		}
		
		public function init():void {
			this.physicalDamage = 1;
			this.life = 48;
			this.maxLife = 48;
			this.healWait = 0;
			this.level = 1;
			this.exp = 0;
			this.killExp = 0;
			this.killGold = 0;
			this.swordLevel = 1;
			this.axeLevel = 1;
			this.shieldLevel = 1;
			this.armor = 32;
			this.physicalBound = PHYSICAL_BOUND;
			this.projectileBound = PROJECTILE_BOUND;
			this.sparkType = "spark2";
		}
		
		public function canAddAura():Boolean {
			return !this.aura;
		}
		
		public function addAura(field:Field, aura:Aura):void {
			this.aura = aura;
			field.audioPlayer.tryStartAudio(Sounds.getInstance().seAura);
		}
		
		public function getAuraType():String {
			return this.aura ? this.aura.type : null;
		}
		
		public function getAuraLevel():int {
			return this.aura ? this.aura.level : 0;
		}
		
		public function gainExp(field:Field, value:int):void {
			value -= this.level;
			if (value < 1) {
				value = 1;
			}
			this.exp += value;
			if (this.exp >= 100) {
				this.exp -= 100;
				this.level++;
				this.maxLife = 40 + level * 8;
				this.quickHealing = true;
				this.invincibleFrames += 64;
				field.addEffect(new LevelUpMessageEffect(), this.position.x, this.position.y);
				for (var i:int = 0; i < 8; i++) {
					field.addEffect(new LevelUpEffect(i), this.position.x, this.position.y);
				}
				field.audioPlayer.tryStartAudio(Sounds.getInstance().seLevelUp);
				//field.messageLayer.message(field, FusalgStrings.getString("levelup", [this.level]));
			}
		}
		
		override public function willDispose():Boolean {
			//Never disposes
			return false;
		}
		
		override public function onGetDamage(field:Field, src:AMortal, damage:int):void {
			this.quickHealing = false;
			if (EDebug.DEBUG_AREA) {
				//invincible patch
			} else {
				if (damage <= 0) {
					return;
				}
				damage -= (this.level + this.shieldLevel) / 2;
				if (this.getMyProjectileDamage() == 1) {
					damage++;
					damage >>= 1;
				}
				if (this.getAuraType() == Aura.AURA_BARRIER) {
					damage++;
					damage >>= 1;
				}
				if (damage < 0) {
					damage = 1;
				}
				super.onGetDamage(field, src, damage);
				if (this.life <= 0) {
					if (EDebug.DEBUG_RESURRECTION) {
						this.life = this.maxLife;
					} else {
						this.life = 0;
						field.initTitle(false, true);
					}
				}
			}
		}

		override public function frameMove(field:Field):void {
			var dx:int = 0;
			var dy:int = 0;
			var speed:int = EDebug.DEBUG_AREA ? 16 : 4;
			if (this.quickHealing) {
				this.healWait += 8;
			}
			switch (this.getAuraType()) {
				case Aura.AURA_ACCEL:
				speed = 6;
				break;
				case Aura.AURA_HEAL:
				this.healWait += this.getAuraLevel();
				break;
			}
			if (this.healWait > 32) {
				this.healWait = 0;
				if (++this.life > this.maxLife) {
					this.life = this.maxLife;
				}
			}
			if (field.keyState.getStateOfKey("RIGHT")) {
				if (this.position.x < 336) {
					dx = speed;
				}
			} else if (field.keyState.getStateOfKey("LEFT")) {
				if (this.position.x > -16) {
					dx = -speed;
				}
			}
			if (field.keyState.getStateOfKey("UP")) {
				dy = -speed;
			} else if (field.keyState.getStateOfKey("DOWN")) {
				if (this.position.y < 394) {
					dy = speed;
				}
			}
			this.run(field, dx, dy);
			if (this.canAttack()) {
				var castdamage:int = (this.getAuraType() == Aura.AURA_POWER) ? (2 + this.getAuraLevel()) : 0;
				if (field.keyState.getStateOfKey("z")) {
					castdamage += this.level + this.swordLevel;
					this.attack(field, "sword", new FusaProjectile(Chips.getInstance().weaponChipM, 8, 4, 64, castdamage));
					field.audioPlayer.tryStartAudio(Sounds.getInstance().seSword);
				} else if (field.keyState.getStateOfKey("x")) {
					castdamage += this.level + this.axeLevel;
					this.attack(field, "axe", new FusaProjectile(Chips.getInstance().weaponChipM, 16, 2, 64, castdamage));
					field.audioPlayer.tryStartAudio(Sounds.getInstance().seAxe);
				} else if (field.keyState.getStateOfKey("c")) {
					this.attack(field, "shield", new FusaProjectile(Chips.getInstance().weaponChipM, 16 + this.shieldLevel, 1, 32 + this.shieldLevel, 1));
					field.audioPlayer.tryStartAudio(Sounds.getInstance().seShield);
				}
			}
			if (this.aura) {
				this.aura.frameMove(field);
				if (this.aura.willDispose()) {
					this.aura = null;
				}
			} else if (field.heroine) {
				var heroinepos:Point = field.heroine.position;
				dx = this.position.x - heroinepos.x;
				dy = this.position.y - heroinepos.y;
				if (dx * dx + dy * dy < 2048) {
					field.heroine.assistAura(field);
				}
			}
			super.frameMove(field);
		}
		
		override public function drawSelf(field:Field):void {
			if (this.aura) {
				this.aura.position.x = this.position.x + FusalgMath.randomInt(-4, 4);
				this.aura.position.y = this.position.y + FusalgMath.randomInt(-4, 4);
				this.aura.drawSelf(field);
			}
			super.drawSelf(field);
		}
	}
	
}
