﻿
package net.kaikoga.fusalg.mortal.boss {
	
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import net.kaikoga.fusalg.chip.Chips;
	import net.kaikoga.fusalg.chip.DotchipChipSet;
	import net.kaikoga.fusalg.field.Field;
	import net.kaikoga.fusalg.mortal.ACharacter;
	import net.kaikoga.fusalg.mortal.AMortal;
	import net.kaikoga.fusalg.mortal.effects.SparkEffectL;
	import net.kaikoga.fusalg.util.FusalgMath;
	
	/**
	 * ...
	 * @author kaikoga
	 */
	public class Hydra extends ACharacter {
		
		private var chipsM:DotchipChipSet;
		private var chipsL:DotchipChipSet;
		
		private static const HEAD_COUNT:int = 4;
		private var heads:Array/*HydraHead*/;
		private var footLeft:BossPart;
		private var footRight:BossPart;
		private var mode:int = 2;
		
		private var dx:int = 0;
		private var dy:int = 4;
		
		public function Hydra() {
			this.chipsM = Chips.getInstance().bossChipM;
			this.chipsL = Chips.getInstance().bossChipL;
			super(this.chipsL);
			this.chipName = "hy_body";
			this.chipShift = new Point( -32, -48);
			this.physicalBound = new Rectangle(-32, -48, 64, 64);
			this.projectileBound = new Rectangle( -48, -64, 96, 96);
			this.physicalDamage = 30;
			this.heads = [];
			for (var i:int = 0; i < HEAD_COUNT; i++) {
				this.heads[i] = new HydraHead(this, this.chipsM, this.chipsL);
			}
			this.footLeft = new BossPart(this, this.chipsM, "hy_foot");
			this.footRight = new BossPart(this, this.chipsM, "hy_foot");
			this.life = 300;
			this.armor = 32;
			this.singleDir = true;
			this.killSparkFrames = 128;
			this.killExp = 0;
			this.killGold = 0;
		}
		
		override public function onAdd(field:Field):void {
			super.onAdd(field);
			field.addPart(this.heads[0], -96, -48);
			field.addPart(this.heads[1], 32, -64);
			field.addPart(this.heads[2], 32, -64);
			field.addPart(this.heads[3], 96, -48);
			for (var i:int = 0; i < HEAD_COUNT; i++) {
				HydraHead(this.heads[i]).addNecks(field);
			}
			field.addPart(this.footLeft, -32, 16);
			field.addPart(this.footRight, 32, 16);
		}
		
		override public function willDispose():Boolean {
			return this.willDisposeByLife();
		}
		
		override public function onKill(field:Field):void {
			super.onKill(field);
			for (var i:int = 0; i < HEAD_COUNT; i++) {
				HydraHead(this.heads[i]).onKill(field);
			}
		}
		
		override public function onDispose(field:Field):void {
			super.onDispose(field);
			for (var i:int = 0; i < HEAD_COUNT; i++) {
				HydraHead(this.heads[i]).dispose();
			}
			this.footLeft.dispose();
			this.footRight.dispose();
		}
		
		override public function onGetDamage(field:Field, src:AMortal, damage:int):void {
			super.onGetDamage(field, src, damage);
			this.resetHeads(false);
		}
		
		private function resetHeads(force:Boolean):void {
			var oldmode:int = this.mode;
			this.mode = this.life / 100;
			if (force || (this.mode != oldmode)) {
				var i:int;
				switch(mode) {
					case 0:
					for (i = 0; i < HEAD_COUNT; i++) {
						HydraHead(this.heads[i]).castsFire = true;
					}
					break;
					case 1:
					for (i = 0; i < HEAD_COUNT; i++) {
						HydraHead(this.heads[i]).castsFire = false;
					}
					HydraHead(this.heads[FusalgMath.randomInt(0, HEAD_COUNT - 1)]).castsFire = true;
					HydraHead(this.heads[FusalgMath.randomInt(0, HEAD_COUNT - 1)]).castsFire = true;
					break;
					case 2:
					for (i = 0; i < HEAD_COUNT; i++) {
						HydraHead(this.heads[i]).castsFire = false;
					}
					break;
				}
			}
		}
		
		override public function frameMove(field:Field):void {
			super.frameMove(field);
			if (Math.random() < 0.05) {
				this.dx = FusalgMath.randomInt( -2, 2);
				this.resetHeads(true);
			} else if (Math.random() < 0.05) {
				this.dy = (this.position.y < 128) ? 2 : -2;
			} else {
				if (this.position.y > 300) {
					this.dy = -2;
				} else if (this.position.y < 20) {
					this.dy = 2;
				}
			}
			this.position.x += this.dx;
			this.position.y += this.dy;
			this.yankBoss(4);
			this.frame++;
			var b:Boolean = Boolean(this.frame & 32);
			this.footLeft.delayMoveRelative(-32, b ? 24 : 8, 0.2);
			this.footRight.delayMoveRelative(32, b ? 8 : 24, 0.2);
		}
		
	}
	
}
