﻿
package net.kaikoga.fusalg.mortal.boss {
	
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import net.kaikoga.fusalg.chip.Chips;
	import net.kaikoga.fusalg.chip.DotchipChipSet;
	import net.kaikoga.fusalg.field.Field;
	import net.kaikoga.fusalg.mortal.ACharacter;
	import net.kaikoga.fusalg.mortal.AMortal;
	import net.kaikoga.fusalg.mortal.bullet.FireBullet;
	import net.kaikoga.fusalg.util.FusalgMath;
	
	/**
	 * ...
	 * @author kaikoga
	 */
	public class Wizard extends ACharacter {
		
		private var chipsM:DotchipChipSet;
		private var chipsL:DotchipChipSet;
		
		private var handLeft:BossPart;
		private var handRight:BossPart;
		
		private var targT:int = 0;
		private var targX:int = 0;
		private var targY:int = 4;
		
		public function Wizard() {
			this.chipsM = Chips.getInstance().bossChipM;
			this.chipsL = Chips.getInstance().bossChipL;
			super(this.chipsL);
			this.chipName = "wi_1";
			this.physicalBound = new Rectangle(-32, -48, 64, 64);
			this.projectileBound = new Rectangle( -48, -64, 96, 96);
			this.chipShift = new Point( -32, -48);
			this.handLeft = new BossPart(this, this.chipsM, "wi_hl");
			this.handRight = new BossPart(this, this.chipsM, "wi_hr");
			this.life = 200;
			this.armor = 48;
			this.singleDir = true;
			this.killSparkFrames = 100;
			this.killExp = 100;
			this.killGold = 2000;
		}
		
		override public function onAdd(field:Field):void {
			super.onAdd(field);
			field.addPart(this.handLeft, -32, 0);
			field.addPart(this.handRight, 32, 0);
		}
		
		override public function onDispose(field:Field):void {
			super.onDispose(field);
			this.handLeft.dispose();
			this.handRight.dispose();
		}
		
		override public function onKill(field:Field):void {
			super.onKill(field);
			this.handLeft.onKill(field);
			this.handRight.onKill(field);
		}
		
		override public function onGetDamage(field:Field, src:AMortal, damage:int):void {
			super.onGetDamage(field, src, damage);
			if (this.life > 0) {
				this.targX = FusalgMath.randomInt(64, 288);
				this.targY = FusalgMath.randomInt(64, 288);
				this.position.x = this.targX;
				this.position.y = this.targY;
			} else {
				this.invincibleFrames = 100;
			}
			this.targT = 16;
		}
		
		override public function frameMove(field:Field):void {
			super.frameMove(field);
			if (this.life <= 0) {
				return;
			}
			this.frame++;
			var casting:Boolean = false;
			if (this.targT > 0) {
				//moving or guarding
				if (this.invincibleFrames < 2) {
					this.invincibleFrames = 2;
				}
				this.chipName = (this.frame & 2) ? "wi_orb1" : "wi_orb2";
				this.targT--;
				this.position.x = ((this.position.x * this.targT) + this.targX) / (this.targT + 1);
				this.position.y = ((this.position.y * this.targT) + this.targY) / (this.targT + 1);
			} else if (this.chipName == "wi_cast") {
				//casting
				casting = (this.frame < 70);
				switch (this.frame & 7) {
					case 0:
					field.addBullet(new FireBullet(8, -40), this.position.x, this.position.y);
					field.addBullet(new FireBullet(8, 0), this.position.x, this.position.y);
					field.addBullet(new FireBullet(8, 40), this.position.x, this.position.y);
					field.addBullet(new FireBullet(6, -30), this.handLeft.position.x, this.handLeft.position.y);
					field.addBullet(new FireBullet(6, -70), this.handLeft.position.x, this.handLeft.position.y);
					field.addBullet(new FireBullet(6, -110), this.handLeft.position.x, this.handLeft.position.y);
					field.addBullet(new FireBullet(6, -150), this.handLeft.position.x, this.handLeft.position.y);
					field.addBullet(new FireBullet(6, 30), this.handRight.position.x, this.handRight.position.y);
					field.addBullet(new FireBullet(6, 70), this.handRight.position.x, this.handRight.position.y);
					field.addBullet(new FireBullet(6, 110), this.handRight.position.x, this.handRight.position.y);
					field.addBullet(new FireBullet(6, 150), this.handRight.position.x, this.handRight.position.y);
					break;
				}
				if (this.frame > 100) {
					this.chipName = "wi_1";
				}
			} else {
				//in no guard
				this.chipName = (this.frame & 4) ? "wi_1" : "wi_2";
				switch (FusalgMath.randomInt(0, 100)) {
					case 0:
					this.targX = FusalgMath.randomInt(64, 288);
					this.targY = FusalgMath.randomInt(64, 288);
					this.targT = 8;
					break;
					case 1:
					case 2:
					this.targX = FusalgMath.randomInt(64, 288);
					this.targY = FusalgMath.randomInt(64, 288);
					this.targT = 16;
					break;
					case 3:
					this.targT = 32;
					break;
					case 4:
					this.targT = 64;
					break;
					case 5:
					case 6:
					this.chipName = "wi_cast";
					this.frame = 0;
					break;
					case 7:
					this.chipName = "wi_cast";
					this.frame = 30;
					break;
				}
			}
			this.yankBoss();
			this.handLeft.delayMoveRelative(casting ? -48 : -32, casting ? 8 : 24, 0.15);
			this.handRight.delayMoveRelative(casting ? 48 : 32, casting ? 8 : 24, 0.15);
		}
		
		override public function drawSelf(field:Field):void {
			super.drawSelf(field);
			
		}
		
	}
	
}
